#include "Config.h"

Config::Config(const char* filepath): _filepath(filepath)
{
    if (!LittleFS.begin())
    {
        Serial.println("An Error has occurred while mounting LittleFS");
        return;
    }
}
Config::~Config()
{
    free(buffer);
}
bool Config::setFilePath(const char* filepath){
    this->_filepath = filepath;
    return loadBin();
}
bool Config::loadBin()
{
    File config_file = LittleFS.open(_filepath, "r");
    //if (buffer == nullptr) // Allocate only once
    //    buffer = (uint8_t *)malloc(CONFIG_SIZE);
    if (config_file.available())
    {
        config_file.read(buffer, CONFIG_SIZE);
        buffer[OFFSET_SSID + 0x1F] = 0x00; // ensure ssid and password are terminated with a null character
        buffer[OFFSET_PASS + 0x1F] = 0x00;
    }
    config_file.close();
    return true;
}

bool Config::saveBin()
{
    File config_file = LittleFS.open(_filepath, "w");
    config_file.write(buffer, CONFIG_SIZE);
    delay(100);
    config_file.close();
    return true;
}
bool Config::setSSID(const char *ssid)
{
    size_t offset = 0;
    bool copy = true;
    for (int i = 0; i < 31; i++)
    {
        if (ssid[i] == 0x00)
            copy = false;
        buffer[offset + i] = copy ? ssid[i] : 0x00;
    }
    buffer[offset + 31] = '\n';
    return true;
}
bool Config::setPASS(const char *pass)
{
    size_t offset = 0x20;
    bool copy = true;
    for (int i = 0; i < 31; i++)
    {
        if (pass[i] == 0x00)
            copy = false;
        buffer[offset + i] = copy ? pass[i] : 0x00;
    }
    buffer[offset + 31] = '\n';
    return true;
}
void Config::print()
{
    Serial.print("BufferHEX ");
    for (int i = 0; i < CONFIG_SIZE; i++)
    {
        Serial.print(i, HEX);
        Serial.print(">");
        Serial.print(buffer[i], HEX);
        Serial.print("|");
    }
    Serial.print("SSID: ");
    Serial.println(SSID);
    Serial.print("PASS: ");
    Serial.println(PASS);
    Serial.print("IP");
    Serial.print(*((uint8_t *)&ip), DEC);
    Serial.print(".");
    Serial.print(* (((uint8_t *)&ip + 1)), DEC);
    Serial.print(".");
    Serial.print(* ((uint8_t *)&ip + 2), DEC);
    Serial.print(".");
    Serial.println( * ((uint8_t *)&ip + 3), DEC);
    Serial.print("GW");
    Serial.print( * ((uint8_t *)&gw), DEC);
    Serial.print(".");
    Serial.print( * ((uint8_t *)&gw + 1), DEC);
    Serial.print(".");
    Serial.print( * ((uint8_t *)&gw + 2), DEC);
    Serial.print(".");
    Serial.println( * ((uint8_t *)&gw + 3), DEC);
    Serial.print("SUBNET");
    Serial.print( * ((uint8_t *)&subnet), DEC);
    Serial.print(".");
    Serial.print( * ((uint8_t *)&subnet + 1), DEC);
    Serial.print(".");
    Serial.print( * ((uint8_t *)&subnet + 2), DEC);
    Serial.print(".");
    Serial.println( * ((uint8_t *)&subnet + 3), DEC);
}