//#define DEBUG
#include <Arduino.h>
#include <Wire.h>
#include "Keyboard.h"
#include "Rfid.h"
#include "Interface.h"
#include <ESP8266WiFi.h>
#include "WebConsole.h"
#include "UserDb.h"
#include "Config.h"
#include "Relais.h"

// File config
Config config("/settings");
userdb::UserDb userdatabase("/userdb.csv");
webconsole::WebConsole web;
// Rfid
Rfid rfid;
// i2C Bus
#define PIN_WIRE_SDA D3
#define PIN_WIRE_SCL D4
Relais relay(D1);
Keyboard keyboard(200);
Interface iface;
void setup()
{
	config.loadBin();
	Serial.begin(115200);
	Serial.println("Starting System...");
#ifdef DEBUG
	userdatabase.print_to_serial();
#endif
	web.init(&config, &userdatabase);
	keyboard.begin(&Wire);
	rfid.begin();
	web.attachRfid(&rfid);
	iface.begin(&keyboard);
}

void loop()
{
	relay.cylce();
	rfid.scan();
	web.serve();
	keyboard.scanAsync();
	userdb::User login_user;
	if (web.isInterceptingRfid() == 3)
		iface.showMessage("WebUI connected", "Awaiting RFID", 2000);
	else if (web.isInterceptingRfid() == 1)
		iface.showMessage("WebUI connected", "RFID Found", 1000);
	else if (iface.pinAvailable() && iface.getState() != Interface::GREET)
	{

		unsigned long delta = millis();
		login_user = userdatabase.user_by_pin(iface.getPin());
		Serial.println("Query duration: " + String(millis() - delta));
		if (login_user.enabled == false)
		{
			iface.showMessage("Login failed!", "-> Pin incorrect", 3000);
			return;
		}
	}
	else if (rfid.available() && iface.getState() != Interface::GREET)
	{
		unsigned long delta = millis();
		login_user = userdatabase.user_by_rfid(rfid.getID());
		Serial.println("Query duration: " + String(millis() - delta));
		if (login_user.enabled == false)
		{
			iface.showMessage("Login failed!", "-> Unkown Card", 3000);
			return;
		}
	}

	if (login_user.enabled == true)
	{
		iface.greetUser(login_user.first_name + " " + login_user.last_name);
		Serial.println("Logon from User " + login_user.toString());
		relay.activate(config.hold_time);
	}
	iface.render();
}